#!/usr/bin/env python
import sys
import os

if not len(sys.argv) >= 3:
   print("Usage: %s license files..." % os.path.basename(sys.argv[0]))
   sys.exit()

licenseFileName = sys.argv[1]
licenseText = ""
with open(licenseFileName, 'r') as f:
    licenseText = f.read()
    licenseText = licenseText[0:licenseText.find('*/')]

files = sys.argv[2:]
for fileName in files:
    with open(fileName, 'r') as f:
        text = f.read()
    oldEnd = text.find('*/')
    if oldEnd == -1:
        oldEnd = 0
    text = licenseText + text[oldEnd:]
    with open(fileName, 'w') as f:
        f.write(text)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-FileCopyrightText: 2014 Denis Steckelmacher <steckdenis@yahoo.fr>
#
# SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL

class Member(object):
    def __init__(self, name):
        """
            Member of classes are identified by a name. A parent name can be
            given and will be used to build the string "Parent.prototype.name"
        """
        self._name = name
        self._full_name = name

    def setParentName(self, name, usePrototype = True):
        if usePrototype:
            self._full_name = '%s.prototype.%s' % (name, self.name())
        else:
            self._full_name = '%s.%s' % (name, self.name())

    def setName(self, name):
        self._name = name

    def name(self):
        return self._name

    def fullName(self):
        return self._full_name

    def print(self):
        print('%s = %s;' % (self.fullName(), self.valueToAssign()))

        if self.valueAfterAssignation() != '':
            print(self.valueAfterAssignation())

    def valueToAssign(self):
        """
            Value to assign to the member, for instance "function (){}"
        """
        return '""';

    def valueAfterAssignation(self):
        """
            Line that will be printed after the assignation. Can be used to call
            a function (therefore giving the type of its parameters.
        """
        return '';

class F(Member):
    def __init__(self, returnValue, name, *args):
        """
            A function has a name, a return value, and arguments. Each argument,
            is a tuple whose first entry is the argument name, and the second
            one its type.

            Note that types are not given using names ("int", "bool", etc), but
            values of the type ("1" for an int, "true" or "false" for a boolean,
            "new X()" for class X, etc).
        """
        Member.__init__(self, name)

        self._return_value = returnValue
        self._args = args

    def print(self):
        if self.name() != '':
            # This function is not a member, no need to assign it to an object
            print(self.valueToAssign())
        else:
            Member.print(self)

    def valueToAssign(self):
        # Define the function
        return 'function %s(%s) { return %s; }' % (
            self.name(),
            ', '.join([arg[0] for arg in self._args]),
            self._return_value
        )

    def valueAfterAssignation(self):
        # Call it, so that its parameters have the correct type
        return '%s(%s);' % (
            self.fullName(),
            ', '.join([arg[1] for arg in self._args])
        )

class Var(Member):
    def __init__(self, type, name):
        """
            A variable has a name and a type
        """
        Member.__init__(self, name)

        self._type = type

    def print(self):
        if self.name() != '':
            # This variable is not a member, declare it using 'var'
            print('var %s = %s;' % (self.name(), self.valueToAssign()))
        else:
            Member.print(self)

    def valueToAssign(self):
        return self._type


class Class(F):
    def __init__(self, name, *args):
        F.__init__(self, '', name, *args)
        self._members = []
        self._prototype = None

    def prototype(self, proto):
        self._prototype = proto
        return self

    def member(self, member):
        self._members.append(member)
        return self

    def members(self, *args):
        for arg in args:
            self.member(arg)

        return self

    def print(self):
        # Declare the constructor (a function)
        print('/*\n * %s\n */' % self.fullName())

        # Always declare a function using the function keyword
        old_full_name = self._full_name
        old_name = self._name

        self._full_name = self._full_name.split('.')[-1]
        self._name = self._full_name

        F.print(self)

        if old_full_name != self._name:
            print('%s = %s;' % (old_full_name, self._name))

        self._full_name = old_full_name
        self._name = old_name

        # Print the prototype
        if self._prototype is not None:
            print('%s.prototype = %s;' % (self.fullName(), self._prototype))

        print('')

        # Declare the members
        for member in self._members:
            member.setParentName(self.fullName())
            member.setName('')
            member.print()
            print('')

class Struct(Var):
    def __init__(self, name):
        Var.__init__(self, '{}', name)
        self._members = []

    def member(self, member):
        self._members.append(member)
        return self

    def members(self, *args):
        for arg in args:
            self.member(arg)

        return self

    def print(self):
        # Declare the object
        print('/*\n * %s\n */' % self.fullName())
        Var.print(self)
        print('')

        # Declare the members
        for member in self._members:
            member.setParentName(self.fullName(), False)
            member.setName('')
            member.print()
            print('')

class Module(object):
    def __init__(self):
        self._members = []

    def member(self, member):
        self._members.append(member)
        return self

    def members(self, *args):
        for arg in args:
            self.member(arg)

        return self

    def print(self):
        # Declare the members in "exports"
        for member in self._members:
            member.setParentName('exports', False)
            member.setName('')
            member.print()
            print('')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-FileCopyrightText: 2014 Denis Steckelmacher <steckdenis@yahoo.fr>
#
# SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL

from jsgenerator import *
from common import *

# Print the license of the generated file (the same as the one of this file)
license()
basicTypes(globals())
require('tls')
require('http')

_function = 'function(){}'
_object = 'new Object()'

Module().members(
    F('new Server()', 'createServer', ('requestListener', _function)),
    Class('Server').prototype('tls.Server').members(
        F(_void, 'listen', ('port', _int), ('hostname', _string), ('backlog', _int), ('callback', _function)),
        F(_void, 'close', ('callback', _function))
    ),
    F('new http.ClientRequest()', 'request', ('options', _object), ('callback', _function)),
    F('new http.ClientRequest()', 'get', ('options', _object), ('callback', _function)),
    Class('Agent').prototype('http.Agent'),
    Var('new Agent()', 'globalAgent')
).print()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-FileCopyrightText: 2014 Denis Steckelmacher <steckdenis@yahoo.fr>
#
# SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL

from jsgenerator import *
from common import *

# Print the license of the generated file (the same as the one of this file)
license()
basicTypes(globals())

_function = 'function(){}'
_object = 'new Object()'

Module().members(
    Class('EventEmitter').members(
        F(_void, 'addListener', ('event', _string), ('listener', _function)),
        F(_void, 'on', ('event', _string), ('listener', _function)),
        F(_void, 'once', ('event', _string), ('listener', _function)),
        F(_void, 'removeListener', ('event', _string), ('listener', _function)),
        F(_void, 'removeAllListeners', ('event', _string)),
        F(_void, 'setMaxListeners', ('n', _int)),
        F(_array, 'listeners', ('event', _string)),
        F(_void, 'emit', ('event', _string), ('args', _mixed)),
        F(_int, 'listenerCount', ('emitter', 'new EventEmitter()'), ('event', _string))
    )
).print()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-FileCopyrightText: 2014 Denis Steckelmacher <steckdenis@yahoo.fr>
#
# SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL

from jsgenerator import *
from common import *

# Print the license of the generated file (the same as the one of this file)
license()
basicTypes(globals())
require('event')

_object = 'new Object()'
_function = 'function(){}'

Module().members(
    Class('Readable').prototype('event.EventEmitter').members(
        F(_string, 'read', ('size', _int)),
        F(_void, 'setEncoding', ('encoding', _string)),
        F(_void, 'resume'),
        F(_void, 'pause'),
        F(_void, 'pipe', ('destination', 'new Writable()'), ('options', _object)),
        F(_void, 'unpipe', ('destination', 'new Writable()')),
        F(_void, 'unshift', ('chunk', _string)),
        F('new Readable()', 'wrap', ('stream', _mixed))
    ),
    Class('Writable').prototype('event.EventEmitter').members(
        F(_void, 'write', ('chunk', _string), ('encoding', _string), ('callback', _function)),
        F(_void, 'end', ('chunk', _string), ('encoding', _string), ('callback', _function))
    ),
    Class('Duplex').prototype('event.EventEmitter').members(
        F(_string, 'read', ('size', _int)),
        F(_void, 'setEncoding', ('encoding', _string)),
        F(_void, 'resume'),
        F(_void, 'pause'),
        F(_void, 'pipe', ('destination', 'new Writable()'), ('options', _object)),
        F(_void, 'unpipe', ('destination', 'new Writable()')),
        F(_void, 'unshift', ('chunk', _string)),
        F('new Readable()', 'wrap', ('stream', _mixed)),
        F(_void, 'write', ('chunk', _string), ('encoding', _string), ('callback', _function)),
        F(_void, 'end', ('chunk', _string), ('encoding', _string), ('callback', _function))
    ),
    Class('Transform').prototype('exports.Duplex')
).print()



import os
import sys

root = os.path.realpath(os.path.dirname(os.path.realpath(__file__)) + "/../../")

if "--getenv" in sys.argv:
    print("KDEROOT=" + root)
    print("PYTHONPATH=" + root + "/lib/site-packages")
    print("PATH=" + root + "/bin:" + os.environ["PATH"])



{% load kdev_filters %}
{% block license_header %}
{% if license %}
#

{{ license|lines_prepend:"# " }}
#
{% endif %}
{% endblock license_header %}


class {{ name }}{% if base_classes %}({% for inh in base_classes %}{{ inh.baseType }}{% if not forloop.last %}, {% endif %}{% endfor %}){% endif %}:
    def __init__(self{% for member in members %}, {{ member.name }}{% endfor %}):
        {% for inh in base_classes %}
        {{ inh.baseType }}.__init__(self)
        {% endfor %}


        {% for member in members %}
        self.{{ member.name }} = {{ member.name }}
        {% empty %}
        pass
        {% endfor %}


    {% for method in methods %}

    {% if method.isStatic %}
    @staticmethod
    def {{ method.name }}({% include "arguments_names.txt" %}):
    {% else %}{% if method.arguments %}
    def {{ method.name }}(self{% for arg in method.arguments %}, {{ arg.name }}{% endfor %}):
    {% else %}
    def {{ method.name }}(self):
    {% endif %}{% endif %}
        pass

    {% endfor %}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-FileCopyrightText: 2014 Denis Steckelmacher <steckdenis@yahoo.fr>
#
# SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL

from jsgenerator import *
from common import *

# Print the license of the generated file (the same as the one of this file)
license()
basicTypes(globals())
require('stream')
require('buffer')

_object = 'new Object()'
_function = 'function(){}'
_options = ('options', _object)
_buf = ('buf', 'new buffer.Buffer()')
_callback = ('callback', _function)

Module().members(
    F('new Gzip()', 'createGzip', _options),
    F('new Gunzip()', 'createGunzip', _options),
    F('new Deflate()', 'createDeflate', _options),
    F('new Inflate()', 'createInflate', _options),
    F('new DeflateRaw()', 'createDeflateRaw', _options),
    F('new InflateRaw()', 'createInflateRaw', _options),
    F('new Unzip()', 'createUnzip', _options),
    Class('Zlib').prototype('stream.Duplex').members(
        F(_void, 'flush', ('callback', _function)),
        F(_void, 'reset')
    ),
    Class('Gzip').prototype('exports.Zlib'),
    Class('Gunzip').prototype('exports.Zlib'),
    Class('Deflate').prototype('exports.Zlib'),
    Class('Inflate').prototype('exports.Zlib'),
    Class('DeflateRaw').prototype('exports.Zlib'),
    Class('InflateRaw').prototype('exports.Zlib'),
    Class('Unzip').prototype('exports.Zlib'),
    F(_void, 'deflate', _buf, _callback),
    F(_void, 'deflateRaw', _buf, _callback),
    F(_void, 'gzip', _buf, _callback),
    F(_void, 'gunzip', _buf, _callback),
    F(_void, 'inflate', _buf, _callback),
    F(_void, 'inflateRaw', _buf, _callback),
    F(_void, 'unzip', _buf, _callback)
).print()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-FileCopyrightText: 2014 Denis Steckelmacher <steckdenis@yahoo.fr>
#
# SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL

from jsgenerator import *
from common import *

# Print the license of the generated file (the same as the one of this file)
license()
basicTypes(globals())
require('stream')
require('buffer')
require('event')

_function = 'function(){}'
_object = 'new Object()'
_date = 'new Date()'
_buffer = 'new buffer.Buffer()'
_stats = 'new Stats'
_callback = ('callback', _function)
_fd = ('fd', _int)
_len = ('len', _int)
_path = ('path', _string)
_uid = ('uid', _int)
_gid = ('gid', _int)
_mode = ('mode', _int)
_offset = ('offset', _int)
_length = ('length', _int)
_position = ('position', _int)
_filename = ('filename', _string)
_options = ('options', _object)
_data = ('data', _string)

Module().members(
    F(_void, 'rename', ('oldPath', _string), ('newPath', _string), _callback),
    F(_int, 'renameSync', ('oldPath', _string), ('newPath', _string)),
    F(_void, 'ftruncate', _fd, _len, _callback),
    F(_int, 'ftruncateSync', _fd, _len),
    F(_void, 'truncate', _path, _len, _callback),
    F(_int, 'truncateSync', _path, _len),
    F(_void, 'chown', _path, _uid, _gid, _callback),
    F(_int, 'chownSync', _path, _uid, _gid),
    F(_void, 'fchown', _fd, _uid, _gid, _callback),
    F(_int, 'fchownSync', _fd, _uid, _gid),
    F(_void, 'lchown', _path, _uid, _gid, _callback),
    F(_int, 'lchownSync', _path, _uid, _gid),
    F(_void, 'chmod', _path, _mode, _callback),
    F(_int, 'chmodSync', _path, _mode),
    F(_void, 'fchmod', _fd, _mode, _callback),
    F(_int, 'fchmodSync', _fd, _mode),
    F(_void, 'lchmod', _path, _mode, _callback),
    F(_int, 'lchmodSync', _path, _mode),
    F(_void, 'stat', _path, _callback),
    F(_void, 'lstat', _path, _callback),
    F(_void, 'fstat', _fd, _callback),
    F(_stats, 'statSync', _path),
    F(_stats, 'lstatSync', _path),
    F(_stats, 'fstatSync', _fd),
    F(_void, 'link', ('srcpath', _string), ('dstpath', _string), _callback),
    F(_int, 'linkSync', ('srcpath', _string), ('dstpath', _string)),
    F(_void, 'symlink', ('srcpath', _string), ('dstpath', _string), ('type', _string), _callback),
    F(_int, 'symlinkSync', ('srcpath', _string), ('dstpath', _string), ('type', _string)),
    F(_void, 'readlink', _path, _callback),
    F(_int, 'readlinkSync', _path),
    F(_void, 'realpath', _path, ('cache', _object), _callback),
    F(_string, 'realpathSync', _path, ('cache', _object)),
    F(_void, 'unlink', _path, _callback),
    F(_int, 'unlinkSync', _path),
    F(_void, 'rmdir', _path, _callback),
    F(_int, 'rmdirSync', _path),
    F(_void, 'mkdir', _path, _mode, _callback),
    F(_int, 'mkdirSync', _path, _mode),
    F(_void, 'readdir', _path, _callback),
    F(_array, 'readdirSync', _path),
    F(_void, 'close', _fd, _callback),
    F(_int, 'closeSync', _fd),
    F(_void, 'open', _path, ('flags', _string), _mode, _callback),
    F(_int, 'openSync', _path, ('flags', _string), _mode),
    F(_void, 'utimes', _path, ('atime', _date), ('mtime', _date), _callback),
    F(_int, 'utimesSync', _path, ('atime', _date), ('mtime', _date)),
    F(_void, 'futimes', _fd, ('atime', _date), ('mtime', _date), _callback),
    F(_int, 'futimesSync', _fd, ('atime', _date), ('mtime', _date)),
    F(_void, 'fsync', _fd, _callback),
    F(_int, 'fsyncSync', _fd),
    F(_void, 'write', _fd, _buffer, _offset, _length, _position, _callback),
    F(_int, 'writeSync', _fd, _buffer, _offset, _length, _position),
    F(_void, 'read', _fd, _buffer, _offset, _length, _position, _callback),
    F(_int, 'readSync', _fd, _buffer, _offset, _length, _position),
    F(_void, 'readFile', _filename, _options, _callback),
    F(_int, 'readFileSync', _filename, _options),
    F(_void, 'writeFile', _filename, _data, _options, _callback),
    F(_int, 'writeFileSync', _filename, _data, _options),
    F(_void, 'appendFile', _filename, _data, _options, _callback),
    F(_int, 'appendFileSync', _filename, _data, _options),
    F(_void, 'watchFile', _filename, _options, _callback),
    F(_void, 'unwatchFile', _filename, _callback),
    F('new FSWatcher()', 'watch', _filename, _options, _callback),
    F(_void, 'exists', _path, _callback),
    F(_bool, 'existsSync', _path),
    Class('Stats').members(
        F(_bool, 'isFile'),
        F(_bool, 'isDirectory'),
        F(_bool, 'isBlockDevice'),
        F(_bool, 'isCharacterDevice'),
        F(_bool, 'isSymbolicLink'),
        F(_bool, 'isFIFO'),
        F(_bool, 'isSocket'),
        Var(_int, 'dev'),
        Var(_int, 'ino'),
        Var(_int, 'mode'),
        Var(_int, 'nlink'),
        Var(_int, 'uid'),
        Var(_int, 'gid'),
        Var(_int, 'rdev'),
        Var(_int, 'size'),
        Var(_int, 'blksize'),
        Var(_int, 'blocks'),
        Var(_date, 'atime'),
        Var(_date, 'mtime'),
        Var(_date, 'ctime')
    ),
    F('new ReadStream()', 'createReadStream', _path, _options),
    Class('ReadStream').prototype('stream.Readable'),
    F('new WriteStream()', 'createWriteStream', _path, _options),
    Class('WriteStream').prototype('stream.Writable'),
    Class('FSWatcher').prototype('event.EventEmitter').members(
        F(_void, 'close')
    )
).print()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# SPDX-FileCopyrightText: 2014 Denis Steckelmacher <steckdenis@yahoo.fr>
#
# SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL

from jsgenerator import *
from common import *

# Print the license of the generated file (the same as the one of this file)
license()
basicTypes(globals())

Module().members(
    F(_string, 'decode', ('string', _string)),
    F(_string, 'encode', ('string', _string)),
    F(_string, 'toUnicode', ('domain', _string)),
    F(_string, 'toASCII', ('domain', _string)),
    Struct('ucs2').members(
        F(_string, 'decode', ('string', _string)),
        F(_string, 'encode', ('codePoints', _array))
    ),
    Var(_string, 'version')
).print()
